#include "ofApp.h"

//--------------------------------------------------------------
void ofApp::setup(){
    
    /** 描画の設定 **/
    ofSetFrameRate(60); // フレームレートを60fpsにする
    ofSetVerticalSync(true); // 垂直同期を有効にする
    ofBackground(50, 50, 50); //背景色を変更する
    
    /** 照明の設定 **/
    ofEnableLighting(); // 照明機能を有効にする
    light.setPosition(200, 300, 50); // 照明の光源位置を設定する
    light.enable(); // 照明を使用可能にする
    cam.setOrientation(ofPoint(-20, 0, 0)); // カメラの向きを設定する
    
    /** OpenGLの設定 **/
    glEnable(GL_DEPTH_TEST); // Zバッファを有効にする
    
    /** Leap Motionの設定 **/
    leap.open(); // Leap Motionに接続する
}

//--------------------------------------------------------------
void ofApp::update(){
    // Leap Motionにより認識された手の情報をvector配列に格納する
    vector <Hand> hands = leap.getLeapHands();
    
    /** 手が1つ以上認識された場合に下記の処理を行う **/
    if(leap.isFrameNew() && hands.size()){
        
        /** 前フレームで取得した各種情報をクリアする **/
        fingerPos.clear(); // 指の位置情報を保持する配列を空にする
        fingerType.clear(); // 指の種類情報を保持する配列を空にする
        palmPos.clear(); // 手のひらの位置情報を保持する配列を空にする
        wristPos.clear(); // 手首の位置情報を保持する配列を空にする
        handType.clear(); // 手の種類情報を保持する配列を空にする
        
        /** 認識範囲とウィンドウの大きさが合うようにスケーリングする **/
        leap.setMappingX(-230, 230, -ofGetWidth()/2, ofGetWidth()/2); // X軸をスケーリングする
        leap.setMappingY(90, 490, -ofGetHeight()/2, ofGetHeight()/2); // Y軸をスケーリングする
        leap.setMappingZ(-150, 150, -200, 200); // Z軸をスケーリングする
        
        // 認識された手の数だけ処理を繰り返す
        for(int i = 0; i < hands.size(); i++){
            
            /** 手の種類（左右）情報を取得する **/
            if(hands[i].isLeft()){ // 左手の場合は1を配列に格納する
                handType.push_back(1);
            }
            else{  // 右手の場合は0を配列に格納する
                handType.push_back(0);
            }
            
            /** 手のひらの情報を取得する **/
            ofVec3f pp = leap.getMappedofPoint(hands[i].palmPosition()); // 手のひらの3次元位置を取得する
            palmPos.push_back(pp); // 手のひらの位置情報を配列に格納する
            
            /** 手首の情報を取得する **/
            ofVec3f wp = leap.getMappedofPoint(hands[i].wristPosition()); // 手首の3次元位置を取得する
            wristPos.push_back(wp); // 手首の位置情報を配列に格納する
                        
            // 認識された手に含まれる指の数だけ処理を繰り返す
            for(int j = 0; j < hands[i].fingers().count(); j++){
                
                /** 指に関する情報を取得する **/
                const Finger &finger = hands[i].fingers()[j]; // 指情報を取得する
                ofVec3f pt = leap.getMappedofPoint(finger.tipPosition()); // 指情報に含まれる指先の3次元位置を取得する
                fingerPos.push_back(pt); // 指先の位置情報を配列に格納する
                int type = finger.type(); // 指情報に含まれる指の種類を取得する
                fingerType.push_back(type); // 指の種類情報を配列に格納する
            }
        }
    }
    
    // 現在のフレームが古くなったことを通知する
    leap.markFrameAsOld();
}

//--------------------------------------------------------------
void ofApp::draw(){
    
    cam.begin(); // カメラを開始する
    
    // 取得した指の数だけ処理を繰り返す
    for(int i = 0; i < fingerPos.size(); i++){
        
        /** 指先の位置に球を描画する **/
        ofSpherePrimitive sphere; // 球のインスタンスを生成する
        sphere.setPosition(fingerPos[i].x, fingerPos[i].y, fingerPos[i].z); // 描画する球の位置を指先の位置に設定する
        int type = fingerType[i]; // 配列から指の種類を取り出す
        
        // 指の種類で色を分ける
        switch(type){
            case 0: // 親指
            ofSetColor(255, 0, 255); // 球の色を変える
            break;
            case 1: // 人差し指
            ofSetColor(255, 0, 0); // 球の色を変える
            break;
            case 2: // 中指
            ofSetColor(0, 255, 0); // 球の色を変える
            break;
            case 3: // 薬指
            ofSetColor(0, 0, 255); // 球の色を変える
            break;
            case 4: // 小指
            ofSetColor(255, 255, 0); // 球の色を変える
            break;
        }
        
        sphere.draw(); // 球を描画する
    }
    
    // 取得した手の数だけ処理を繰り返す
    for(int i = 0; i < palmPos.size(); i++){
        
        /** 手のひらを描画する **/
        ofSpherePrimitive sphere; // 球のインスタンスを生成する
        sphere.setPosition(palmPos[i].x, palmPos[i].y, palmPos[i].z);  // 描画する球の位置を手首の位置に設定する
        if(handType[i] == 1){ // 左手の場合
            ofSetColor(135, 0, 204); // 球の色を変える
        }
        else{ // 右手の場合
            ofSetColor(204, 0, 135); // 球の色を変える
        }
        sphere.draw(); // 球を描画する
        
        /** 手首を描画する **/
        sphere.setPosition(wristPos[i].x, wristPos[i].y, wristPos[i].z);  // 描画する球の位置を手首の位置に設定する
        ofSetColor(127, 127, 127); // 球の色を変える
        sphere.draw(); // 球を描画する
    }
    
    cam.end(); // カメラを終了する
}

//--------------------------------------------------------------
void ofApp::keyPressed(int key){
    
}

//--------------------------------------------------------------
void ofApp::keyReleased(int key){
    
}

//--------------------------------------------------------------
void ofApp::mouseMoved(int x, int y ){
    
}

//--------------------------------------------------------------
void ofApp::mouseDragged(int x, int y, int button){
    
}

//--------------------------------------------------------------
void ofApp::mousePressed(int x, int y, int button){
    
}

//--------------------------------------------------------------
void ofApp::mouseReleased(int x, int y, int button){
    
}

//--------------------------------------------------------------
void ofApp::mouseEntered(int x, int y){
    
}

//--------------------------------------------------------------
void ofApp::mouseExited(int x, int y){
    
}

//--------------------------------------------------------------
void ofApp::windowResized(int w, int h){
    
}

//--------------------------------------------------------------
void ofApp::gotMessage(ofMessage msg){
    
}

//--------------------------------------------------------------
void ofApp::dragEvent(ofDragInfo dragInfo){
    
}
